'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var axios = require('axios');
var https = require('https');
var dateFns = require('date-fns');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);
var https__default = /*#__PURE__*/_interopDefaultLegacy(https);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

class JumboObject {
    constructor(jumbo, authRequired) {
        this.jumbo = jumbo;
        this.authRequired = authRequired;
    }
}

class Product extends JumboObject {
    /**
     * Gets product from ID
     * @param productId Product ID
     */
    getProductFromId(productId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`products/${productId}`, headers, query);
        });
    }
    /**
     * Get products from given product name
     * @param productName Product name to search for
     * @param offset Offset in search (default 0)
     * @param limit Amount of products returned (default 10)
     * @param filters Any extra filters (in the form of a number)
     * @param sort Sort options as defined in ./productModel
     */
    getProductsFromName(productName, offset, limit, filters, sort, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            // First get query results as productQueryModel
            const products = yield this.jumbo.get(`search`, headers, Object.assign({ q: productName, offset: (offset ? offset : 0).toString(), limit: (limit ? limit : 10).toString(), filters: (filters ? filters : '').toString(), sort: (sort ? sort : '').toString() }, query));
            // Then create a ProductModel for every ProductQuery product
            const result = [];
            for (var key in products.products.data) {
                const product = {
                    product: {
                        data: products.products.data[key],
                    },
                };
                result.push(product);
            }
            // Return array of ProductModels
            return result;
        });
    }
    /**
     * Shortcut function to get the first product when searching for name
     * @param productName Product name to search for
     * @param sort Sort options as defined in ./productModel
     */
    getFirstProductFromName(productName, sort, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const product = yield this.getProductsFromName(productName, 0, 1, undefined, sort, headers, query);
            return product[0];
        });
    }
}

class Store extends JumboObject {
    /**
     * Gets store from store ID
     * @param storeId Store ID
     */
    getStoreFromId(storeId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jumbo.get(`stores/${storeId}`, headers, query);
        });
    }
    /**
     * Find stores based on given longitude, latitude and distance (optional)
     * @param long Longitude
     * @param lat Latitude
     * @param distance Maximum search radius (default none)
     * @param count Number of stores to retrieve (default 20)
     * @param offset Offset in search (default 0)
     */
    getStoresFromLongLat(long, lat, distance, count, offset, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const stores = yield this.jumbo.get(`stores`, headers, Object.assign({ longitude: long.toString(), latitude: lat.toString(), distance: (distance ? distance : '').toString(), count: (count ? count : 20).toString(), offset: (offset ? offset : 0).toString() }, query));
            const result = [];
            for (var key in stores.stores.data) {
                const store = {
                    store: {
                        data: stores.stores.data[key],
                    },
                };
                result.push(store);
            }
            return result;
        });
    }
    /**
     * Shortcut function to get the nearest store given longitude and latitude
     * @param long Longitude
     * @param lat Latitude
     */
    getNearestStoreFromLongLat(long, lat, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const stores = yield this.getStoresFromLongLat(long, lat, undefined, undefined, undefined, headers, query);
            return stores[0];
        });
    }
}

class TokenHandler {
    constructor(jumbo, username, password) {
        this.jumbo = jumbo;
        this.username = username;
        this.password = password;
        this.Ready = new Promise((resolve, reject) => {
            this.generateToken()
                .then(() => {
                resolve(undefined);
            })
                .catch(reject);
        });
    }
    /**
     * Function that can be called to generate token (if there is no token yet)
     */
    generateToken() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.jumboToken) {
                this.jumboToken = yield this.login();
            }
        });
    }
    /**
     * Function that is used to login to Jumbo and get a token
     */
    login() {
        return __awaiter(this, void 0, void 0, function* () {
            // To get a token, we simply initialize a POST request to /users/login with the info
            let body = {
                password: this.password,
                username: this.username,
            };
            // Send POST request with info
            let response = yield this.jumbo.post(`users/login`, JSON.stringify(body), undefined, undefined, false, true);
            // Save token
            return yield response.headers['x-jumbo-token'];
        });
    }
    /**
     * Simple function that returns the token (assuming it exists)
     */
    getToken() {
        // TODO: refresh token if needed
        return this.jumboToken;
    }
}

class Order extends JumboObject {
    /**
     * Gets order from ID
     * @param orderId Order ID
     */
    getMyOrderById(orderId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get order from endpoint
            const order = yield this.jumbo.get(`users/me/orders/${orderId}`, headers, query, this.authRequired);
            // Convert times to dates
            return this.convertOrderResponseToOrderModel(order.order.data);
        });
    }
    /**
     * Returns all the users' orders
     * @param offset Offset (default 0)
     * @param count Amount of orders (default 10)
     */
    getMyOrders(offset, count, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            // Query all orders as orderQueryModel
            const orders = yield this.jumbo.get(`users/me/orders`, headers, Object.assign({ offset: (offset ? offset : 0).toString(), count: (count ? count : 10).toString() }, query), this.authRequired);
            // Convert responses to models
            return orders.orders.data.map((orderResponse) => {
                return this.convertOrderResponseToOrderModel(orderResponse);
            });
        });
    }
    /**
     * Shortcut function to return latest (first) order
     */
    getMyLatestOrder(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(0, 1, headers, query);
            return orders[0];
        });
    }
    /**
     * Returns all of the user's orders with a certain status
     * @param status Status of the order (Processing, Open, Completed)
     */
    getMyOrdersByStatus(status, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(undefined, undefined, headers, query);
            return orders.filter((order) => {
                return order.order.data.status === status;
            });
        });
    }
    /**
     * Shortcut function to retrieve the latest order that matches the status
     * @param status Status of the order
     */
    getMyLatestOrderByStatus(status, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrdersByStatus(status, headers, query);
            return orders[0];
        });
    }
    /**
     * Gets the user's relevant orders (which includes shipping time)
     */
    getMyRelevantOrders(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(undefined, undefined, headers, Object.assign({ relevant: 'true' }, query));
            return orders;
        });
    }
    /**
     * Converts response to actual data by first converting unix timestamps to dates
     * @param order OrderResponse to convert
     */
    convertOrderResponseToOrderModel(order) {
        // Simply convert the delivery object and cut-off date
        const orderModel = {
            order: {
                data: Object.assign(Object.assign({}, order), { delivery: this.convertDeliveryTimesToDates(order.delivery), orderCutOffDate: this.convertUnixDateToDate(order.orderCutOffDate), shipping: undefined }),
            },
        };
        // Add shipping if required
        if (order.shipping) {
            orderModel.order.data.shipping = this.convertShippingTimesToDates(order.shipping);
        }
        return orderModel;
    }
    /**
     * Converts shipping unix timestamps of order to actual dates
     * @param shipping order.data.shipping
     */
    convertShippingTimesToDates(shipping) {
        const shippingDate = {
            plannedETAStart: this.convertUnixDateToDate(shipping.plannedETAStart),
            plannedETAEnd: this.convertUnixDateToDate(shipping.plannedETAEnd),
            unknownLiveETA: shipping.unknownLiveETA,
        };
        if (shipping.liveETA) {
            shippingDate.liveETA = this.convertUnixDateToDate(shipping.liveETA);
        }
        return shippingDate;
    }
    /**
     * Converts delivery unix timestamps of order to actual dates
     * @param delivery order.data.delivery
     */
    convertDeliveryTimesToDates(delivery) {
        // First convert these 3 separately
        const date = this.convertUnixDateToDate(delivery.date);
        const startDate = this.convertUnixDateToDate(delivery.startDateTime);
        const endDate = this.convertUnixDateToDate(delivery.endDateTime);
        // Add them to return object
        const deliveryDate = {
            address: delivery.address,
            date: date,
            endDateTime: endDate,
            location: delivery.location,
            price: delivery.price,
            startDateTime: startDate,
            time: delivery.time,
        };
        // Only convert and add collection date if it's in the original delivery data
        if (delivery.collectionDateTime) {
            const collectionDate = this.convertUnixDateToDate(delivery.collectionDateTime);
            deliveryDate.collectionDateTime = collectionDate;
        }
        return deliveryDate;
    }
    /**
     * Converts given unix timestamp to a Date
     */
    convertUnixDateToDate(unixDate) {
        return dateFns.fromUnixTime(this.formatUnixTimestamp(unixDate));
    }
    /**
     * Strips the last 3 digits off the timestamp to properly convert
     */
    formatUnixTimestamp(unixDate) {
        return (unixDate - (unixDate % 1000)) / 1000;
    }
}
exports.OrderStatus = void 0;
(function (OrderStatus) {
    OrderStatus["Processing"] = "PROCESSING";
    OrderStatus["Open"] = "OPEN";
    OrderStatus["ReadyToDeliver"] = "READY_TO_DELIVER";
    OrderStatus["Completed"] = "PICKED_UP";
})(exports.OrderStatus || (exports.OrderStatus = {}));

class Promotion extends JumboObject {
    /**
     * Get all promotions from a given store (by ID)
     * @param storeId Store ID
     */
    getPromotionsFromStore(storeId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`promotion-overview`, headers, Object.assign({ store_id: storeId.toString() }, query));
        });
    }
}

class Recipe extends JumboObject {
    /**
     * Get recipe from ID
     * @param recipeId Recipe ID
     */
    getRecipeFromId(recipeId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`recipes/${recipeId}`, headers, query);
        });
    }
    /**
     * Get recipes from given recipe name
     * @param recipeName Recipe name to search for
     * @param offset Offset in search (default 0)
     * @param count Amount of recipes returned (default 10)
     */
    getRecipesFromName(recipeName, offset, count, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const recipes = yield this.jumbo.get(`recipes`, headers, Object.assign({ q: recipeName, offset: (offset ? offset : 0).toString(), limit: (count ? count : 10).toString() }, query));
            const result = [];
            for (var key in recipes.recipes.data) {
                const recipe = {
                    recipe: {
                        data: recipes.recipes.data[key],
                    },
                };
                result.push(recipe);
            }
            return result;
        });
    }
    /**
     * Shortcut function to get the first recipe when searching for name
     * @param recipeName Recipe name to search for
     */
    getFirstRecipeFromName(recipeName, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const recipe = yield this.getRecipesFromName(recipeName, 0, 1, headers, query);
            return recipe[0];
        });
    }
    /**
     * Get recipes from given filter ID
     * @param filterId Recipe filter ID
     * @param offset Offset in search (default 0)
     * @param count Amount of recipes returned (default 10)
     */
    getRecipesFromFilterId(filterId, offset, count, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const recipes = yield this.jumbo.get(`recipes`, headers, Object.assign({ filterId: filterId.toString(), offset: (offset ? offset : 0).toString(), limit: (count ? count : 10).toString() }, query));
            const result = [];
            for (var key in recipes.recipes.data) {
                const recipe = {
                    recipe: {
                        data: recipes.recipes.data[key],
                    },
                };
                result.push(recipe);
            }
            return result;
        });
    }
}

class User extends JumboObject {
    /**
     * Returns info of logged in user
     */
    getMyInfo(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`users/me`, headers, query, this.authRequired);
        });
    }
}

class Category extends JumboObject {
    /**
     * Get category from category ID
     * @param categoryId Category ID
     */
    getCategoryFromId(categoryId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.jumbo.get(`categories`, headers, Object.assign({ id: categoryId.toString() }, query));
        });
    }
}

class List extends JumboObject {
    /**
     * Returns all of your lists (login required)
     */
    getMyLists(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/mylists`, headers, query, true);
        });
    }
    /**
     * Returns a single list given a list ID
     * @param listId ID of list
     */
    getListFromId(listId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/${listId}`, headers, query, false);
        });
    }
    /**
     * Returns a list of lists that match the given name (name is case sensitive and very specific)
     * @param listName Name of list (category)
     * @param offset Offset in search (default 0)
     * @param limit Amount of lists to retrieve (default 10)
     */
    getListsByName(listName, offset, limit, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/search`, headers, Object.assign({ offset: (offset ? offset : 0).toString(), limit: (limit ? limit : 10).toString(), q: listName }, query), false);
        });
    }
    /**
     * Shortcut function to get the most popular lists
     * @param offset Offset in search (default 0)
     * @param limit Amount of lists to retrieve (default 10)
     */
    getPopularLists(offset, limit, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getListsByName('', offset, limit, headers, query);
        });
    }
    /**
     * Gets all items from a given list
     * @param listId ID of list
     * @param offset Offset in search (default 0)
     * @param limit Amount of items to retrieve (default 10)
     */
    getItemsFromList(listId, offset, limit, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/${listId}/items`, headers, Object.assign({ offset: (offset ? offset : 0).toString(), limit: (limit ? limit : 10).toString() }, query), false);
        });
    }
    /**
     * Gets all smart lists (i.e. last purchased) of user (login required)
     */
    getMySmartLists(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`users/me/smart-lists`, headers, query, true);
        });
    }
    /**
     * Gets all the lists the user currently follows (login required)
     */
    getMyFollowedLists(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.get(`lists/following`, headers, query, true);
        });
    }
    /**
     * Checks whether the user is following the given list (login required)
     * @param listId ID of list
     */
    isFollowingList(listId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const isFollowing = yield this.jumbo.get(`lists/${listId}/following`, headers, query, true);
            return isFollowing.isFollowing;
        });
    }
    /**
     * Makes the user follow the given list (login required)
     * @param listId ID of list to follow
     */
    followList(listId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.put(`lists/${listId}/follow`, undefined, headers, query, true);
        });
    }
    /**
     * Makes the user unfollow the given list (login required)
     * @param listId ID of list to unfollow
     */
    unfollowList(listId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jumbo.put(`lists/${listId}/unfollow`, undefined, headers, query, true);
        });
    }
}

const endpoint = 'https://mobileapi.jumbo.com/v12/';
class Jumbo {
    /**
     * @param username Jumbo username
     * @param password Jumbo password
     * @param verbose Whether requests should be logged in the console
     * @param config Custom Axios config, must use 'TLSv1.2' as TLS version
     */
    constructor(username, password, verbose, config) {
        this.username = username;
        this.password = password;
        this.verbose = verbose;
        this.config = config;
        // Create https agent for TLSv1.2 or less (API doesn't respond to TLSv1.3+)
        this.client = config
            ? axios__default['default'].create(config)
            : axios__default['default'].create({
                httpsAgent: new https__default['default'].Agent({
                    maxVersion: 'TLSv1.2',
                }),
            });
        // Set separate classes
        this.jumboCategory = new Category(this, false);
        this.jumboList = new List(this, false);
        this.jumboOrder = new Order(this, true);
        this.jumboProduct = new Product(this, false);
        this.jumboPromotion = new Promotion(this, false);
        this.jumboRecipe = new Recipe(this, false);
        this.jumboStore = new Store(this, false);
        this.jumboUser = new User(this, true);
        // Login using given username and password
        if (username && password) {
            this.login(username, password);
        }
    }
    category() {
        return this.jumboCategory;
    }
    list() {
        return this.jumboList;
    }
    order() {
        return this.jumboOrder;
    }
    product() {
        return this.jumboProduct;
    }
    promotion() {
        return this.jumboPromotion;
    }
    recipe() {
        return this.jumboRecipe;
    }
    store() {
        return this.jumboStore;
    }
    user() {
        return this.jumboUser;
    }
    /**
     * Function that creates a new TokenHandler for given username and password
     * @param username Jumbo account username (e-mail)
     * @param password Jumbo account password
     */
    login(username, password) {
        this.tokenHandler = new TokenHandler(this, username, password);
    }
    /**
     * PUT request
     * @param path Endpoint URL (without start)
     * @param body Body of PUT (if any)
     * @param extraHeaders Any extra headers
     * @param query Any query options
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    put(path, body, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, exports.requestMethod.PUT, body, extraHeaders, query, authRequired, fullResponse);
        });
    }
    /**
     * POST request
     * @param path Endpoint URL (without start)
     * @param body Body of POST
     * @param extraHeaders Any extra headers
     * @param query Any query options
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    post(path, body, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, exports.requestMethod.POST, body, extraHeaders, query, authRequired, fullResponse);
        });
    }
    /**
     * GET request
     * @param path Endpoint URL (without start)
     * @param extraHeaders Any extra headers
     * @param query Any query options
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    get(path, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, exports.requestMethod.GET, undefined, extraHeaders, query, authRequired, fullResponse);
        });
    }
    /**
     * Generic request method
     * @param path Endpoint URL (without start)
     * @param method Request method (GET, POST, PUT, DELETE)
     * @param body Body in case of POST and PUT
     * @param extraHeaders Any extra headers (Content-Type and Accept) are already included
     * @param query Query in case of GET
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    request(path, method, body, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            // If auth is required and we don't have a token yet, we should create one
            if (authRequired) {
                if (!this.tokenHandler) {
                    throw new Error(`You must be logged in to access this path: ${endpoint + path}`);
                }
                else {
                    // If the tokenHandler doesn't have a token yet, make sure it gets one
                    yield this.tokenHandler.Ready;
                }
            }
            // Create initial header properties
            let requestHeaders = this.createHeader(authRequired, extraHeaders);
            // Add query to URL if given
            let url = this.createURL(path, query);
            // Log if verbose
            if (this.verbose) {
                console.log(url);
                console.log(method);
                console.log(requestHeaders);
                void (body && console.log(body));
            }
            // Make request
            let response = yield this.client.request({
                method: method,
                url: url,
                headers: requestHeaders,
                data: body,
            });
            // Throw error if response not ok
            if (!response.statusText) {
                const text = response.data;
                throw new Error(`${response.statusText}: ${text}`);
            }
            if (fullResponse) {
                return response;
            }
            // Return response in JSON format
            return response.data;
        });
    }
    /**
     * Helper function to create headers for request
     * @param extraHeaders Any extra header options
     */
    createHeader(authRequired, extraHeaders) {
        // Create header
        let headers = Object.assign({ 'Content-Type': 'application/json', 'User-Agent': 'jumbo-wrapper' }, extraHeaders);
        if (authRequired && this.tokenHandler) {
            headers['x-jumbo-token'] = this.tokenHandler.getToken();
        }
        else if (authRequired && !this.tokenHandler) {
            throw new Error('You must be logged in to use this function');
        }
        // Return the headers
        return headers;
    }
    /**
     * Helper function to create request URL
     * @param path Path to endpoint (without ENDPOINT in .env)
     * @param query Any query options
     */
    createURL(path, query) {
        let url;
        // Add query if given
        if (query) {
            const params = new URLSearchParams(query);
            url = endpoint + path + '?' + params;
        }
        else {
            url = endpoint + path;
        }
        // Return URL
        return url;
    }
}
/**
 * Simple enum for different request methods
 */
exports.requestMethod = void 0;
(function (requestMethod) {
    requestMethod["GET"] = "GET";
    requestMethod["POST"] = "POST";
    requestMethod["PUT"] = "PUT";
})(exports.requestMethod || (exports.requestMethod = {}));

/**
 * The different sort options for products
 */
exports.ProductSortOptions = void 0;
(function (ProductSortOptions) {
    ProductSortOptions["POPULAR"] = "P_Sales|1";
    ProductSortOptions["PRICE_ASC"] = "P_Price|0";
    ProductSortOptions["PRICE_DESC"] = "P_Price|1";
    ProductSortOptions["ALPHAB_ASC"] = "P_Title|0";
    ProductSortOptions["ALPHAB_DESC"] = "P_Title|1";
    ProductSortOptions["PER_WEIGHT_MEASURE"] = "pricePerWeightMeasure|0";
})(exports.ProductSortOptions || (exports.ProductSortOptions = {}));

exports.Category = Category;
exports.Jumbo = Jumbo;
exports.List = List;
exports.Order = Order;
exports.Product = Product;
exports.Promotion = Promotion;
exports.Recipe = Recipe;
exports.Store = Store;
exports.User = User;
