var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from 'axios';
import { Product } from './product/product';
import { Store } from './store/store';
import { TokenHandler } from './auth/tokenHandler';
import https from 'https';
import { Order } from './order/order';
import { Promotion } from './promotion/promotion';
import { Recipe } from './recipe/recipe';
import { User } from './user/user';
import { Category } from './category/category';
import { List } from './list/list';
const endpoint = 'https://mobileapi.jumbo.com/v12/';
export class Jumbo {
    /**
     * @param username Jumbo username
     * @param password Jumbo password
     * @param verbose Whether requests should be logged in the console
     * @param config Custom Axios config, must use 'TLSv1.2' as TLS version
     */
    constructor(username, password, verbose, config) {
        this.username = username;
        this.password = password;
        this.verbose = verbose;
        this.config = config;
        // Create https agent for TLSv1.2 or less (API doesn't respond to TLSv1.3+)
        this.client = config
            ? axios.create(config)
            : axios.create({
                httpsAgent: new https.Agent({
                    maxVersion: 'TLSv1.2',
                }),
            });
        // Set separate classes
        this.jumboCategory = new Category(this, false);
        this.jumboList = new List(this, false);
        this.jumboOrder = new Order(this, true);
        this.jumboProduct = new Product(this, false);
        this.jumboPromotion = new Promotion(this, false);
        this.jumboRecipe = new Recipe(this, false);
        this.jumboStore = new Store(this, false);
        this.jumboUser = new User(this, true);
        // Login using given username and password
        if (username && password) {
            this.login(username, password);
        }
    }
    category() {
        return this.jumboCategory;
    }
    list() {
        return this.jumboList;
    }
    order() {
        return this.jumboOrder;
    }
    product() {
        return this.jumboProduct;
    }
    promotion() {
        return this.jumboPromotion;
    }
    recipe() {
        return this.jumboRecipe;
    }
    store() {
        return this.jumboStore;
    }
    user() {
        return this.jumboUser;
    }
    /**
     * Function that creates a new TokenHandler for given username and password
     * @param username Jumbo account username (e-mail)
     * @param password Jumbo account password
     */
    login(username, password) {
        this.tokenHandler = new TokenHandler(this, username, password);
    }
    /**
     * PUT request
     * @param path Endpoint URL (without start)
     * @param body Body of PUT (if any)
     * @param extraHeaders Any extra headers
     * @param query Any query options
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    put(path, body, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, requestMethod.PUT, body, extraHeaders, query, authRequired, fullResponse);
        });
    }
    /**
     * POST request
     * @param path Endpoint URL (without start)
     * @param body Body of POST
     * @param extraHeaders Any extra headers
     * @param query Any query options
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    post(path, body, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, requestMethod.POST, body, extraHeaders, query, authRequired, fullResponse);
        });
    }
    /**
     * GET request
     * @param path Endpoint URL (without start)
     * @param extraHeaders Any extra headers
     * @param query Any query options
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    get(path, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.request(path, requestMethod.GET, undefined, extraHeaders, query, authRequired, fullResponse);
        });
    }
    /**
     * Generic request method
     * @param path Endpoint URL (without start)
     * @param method Request method (GET, POST, PUT, DELETE)
     * @param body Body in case of POST and PUT
     * @param extraHeaders Any extra headers (Content-Type and Accept) are already included
     * @param query Query in case of GET
     * @param authRequired Whether a token is required for the function
     * @param fullResponse Returns response + headers instead of only data
     */
    request(path, method, body, extraHeaders, query, authRequired, fullResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            // If auth is required and we don't have a token yet, we should create one
            if (authRequired) {
                if (!this.tokenHandler) {
                    throw new Error(`You must be logged in to access this path: ${endpoint + path}`);
                }
                else {
                    // If the tokenHandler doesn't have a token yet, make sure it gets one
                    yield this.tokenHandler.Ready;
                }
            }
            // Create initial header properties
            let requestHeaders = this.createHeader(authRequired, extraHeaders);
            // Add query to URL if given
            let url = this.createURL(path, query);
            // Log if verbose
            if (this.verbose) {
                console.log(url);
                console.log(method);
                console.log(requestHeaders);
                void (body && console.log(body));
            }
            // Make request
            let response = yield this.client.request({
                method: method,
                url: url,
                headers: requestHeaders,
                data: body,
            });
            // Throw error if response not ok
            if (!response.statusText) {
                const text = response.data;
                throw new Error(`${response.statusText}: ${text}`);
            }
            if (fullResponse) {
                return response;
            }
            // Return response in JSON format
            return response.data;
        });
    }
    /**
     * Helper function to create headers for request
     * @param extraHeaders Any extra header options
     */
    createHeader(authRequired, extraHeaders) {
        // Create header
        let headers = Object.assign({ 'Content-Type': 'application/json', 'User-Agent': 'jumbo-wrapper' }, extraHeaders);
        if (authRequired && this.tokenHandler) {
            headers['x-jumbo-token'] = this.tokenHandler.getToken();
        }
        else if (authRequired && !this.tokenHandler) {
            throw new Error('You must be logged in to use this function');
        }
        // Return the headers
        return headers;
    }
    /**
     * Helper function to create request URL
     * @param path Path to endpoint (without ENDPOINT in .env)
     * @param query Any query options
     */
    createURL(path, query) {
        let url;
        // Add query if given
        if (query) {
            const params = new URLSearchParams(query);
            url = endpoint + path + '?' + params;
        }
        else {
            url = endpoint + path;
        }
        // Return URL
        return url;
    }
}
/**
 * Simple enum for different request methods
 */
export var requestMethod;
(function (requestMethod) {
    requestMethod["GET"] = "GET";
    requestMethod["POST"] = "POST";
    requestMethod["PUT"] = "PUT";
})(requestMethod || (requestMethod = {}));
