var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { JumboObject } from '../base/jumboObject';
import { fromUnixTime } from 'date-fns';
export class Order extends JumboObject {
    /**
     * Gets order from ID
     * @param orderId Order ID
     */
    getMyOrderById(orderId, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get order from endpoint
            const order = yield this.jumbo.get(`users/me/orders/${orderId}`, headers, query, this.authRequired);
            // Convert times to dates
            return this.convertOrderResponseToOrderModel(order.order.data);
        });
    }
    /**
     * Returns all the users' orders
     * @param offset Offset (default 0)
     * @param count Amount of orders (default 10)
     */
    getMyOrders(offset, count, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            // Query all orders as orderQueryModel
            const orders = yield this.jumbo.get(`users/me/orders`, headers, Object.assign({ offset: (offset ? offset : 0).toString(), count: (count ? count : 10).toString() }, query), this.authRequired);
            // Convert responses to models
            return orders.orders.data.map((orderResponse) => {
                return this.convertOrderResponseToOrderModel(orderResponse);
            });
        });
    }
    /**
     * Shortcut function to return latest (first) order
     */
    getMyLatestOrder(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(0, 1, headers, query);
            return orders[0];
        });
    }
    /**
     * Returns all of the user's orders with a certain status
     * @param status Status of the order (Processing, Open, Completed)
     */
    getMyOrdersByStatus(status, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(undefined, undefined, headers, query);
            return orders.filter((order) => {
                return order.order.data.status === status;
            });
        });
    }
    /**
     * Shortcut function to retrieve the latest order that matches the status
     * @param status Status of the order
     */
    getMyLatestOrderByStatus(status, headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrdersByStatus(status, headers, query);
            return orders[0];
        });
    }
    /**
     * Gets the user's relevant orders (which includes shipping time)
     */
    getMyRelevantOrders(headers, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const orders = yield this.getMyOrders(undefined, undefined, headers, Object.assign({ relevant: 'true' }, query));
            return orders;
        });
    }
    /**
     * Converts response to actual data by first converting unix timestamps to dates
     * @param order OrderResponse to convert
     */
    convertOrderResponseToOrderModel(order) {
        // Simply convert the delivery object and cut-off date
        const orderModel = {
            order: {
                data: Object.assign(Object.assign({}, order), { delivery: this.convertDeliveryTimesToDates(order.delivery), orderCutOffDate: this.convertUnixDateToDate(order.orderCutOffDate), shipping: undefined }),
            },
        };
        // Add shipping if required
        if (order.shipping) {
            orderModel.order.data.shipping = this.convertShippingTimesToDates(order.shipping);
        }
        return orderModel;
    }
    /**
     * Converts shipping unix timestamps of order to actual dates
     * @param shipping order.data.shipping
     */
    convertShippingTimesToDates(shipping) {
        const shippingDate = {
            plannedETAStart: this.convertUnixDateToDate(shipping.plannedETAStart),
            plannedETAEnd: this.convertUnixDateToDate(shipping.plannedETAEnd),
            unknownLiveETA: shipping.unknownLiveETA,
        };
        if (shipping.liveETA) {
            shippingDate.liveETA = this.convertUnixDateToDate(shipping.liveETA);
        }
        return shippingDate;
    }
    /**
     * Converts delivery unix timestamps of order to actual dates
     * @param delivery order.data.delivery
     */
    convertDeliveryTimesToDates(delivery) {
        // First convert these 3 separately
        const date = this.convertUnixDateToDate(delivery.date);
        const startDate = this.convertUnixDateToDate(delivery.startDateTime);
        const endDate = this.convertUnixDateToDate(delivery.endDateTime);
        // Add them to return object
        const deliveryDate = {
            address: delivery.address,
            date: date,
            endDateTime: endDate,
            location: delivery.location,
            price: delivery.price,
            startDateTime: startDate,
            time: delivery.time,
        };
        // Only convert and add collection date if it's in the original delivery data
        if (delivery.collectionDateTime) {
            const collectionDate = this.convertUnixDateToDate(delivery.collectionDateTime);
            deliveryDate.collectionDateTime = collectionDate;
        }
        return deliveryDate;
    }
    /**
     * Converts given unix timestamp to a Date
     */
    convertUnixDateToDate(unixDate) {
        return fromUnixTime(this.formatUnixTimestamp(unixDate));
    }
    /**
     * Strips the last 3 digits off the timestamp to properly convert
     */
    formatUnixTimestamp(unixDate) {
        return (unixDate - (unixDate % 1000)) / 1000;
    }
}
export var OrderStatus;
(function (OrderStatus) {
    OrderStatus["Processing"] = "PROCESSING";
    OrderStatus["Open"] = "OPEN";
    OrderStatus["ReadyToDeliver"] = "READY_TO_DELIVER";
    OrderStatus["Completed"] = "PICKED_UP";
})(OrderStatus || (OrderStatus = {}));
